/**
 * ArrayList.java
 * 2/2/11
 */

import java.util.Random;

/**
 * Implements an expandable list of Objects.
 * 
 * @author CPSC170A
 * @param <T> type of objects in this array list
 */
public class ArrayList<T> {

    /** Initial size of the array list. */
    private static final int INIT_SIZE = 10;

    /** Minimum capacity fraction. */
    private static final double MIN_CAPACITY = 0.25;

    /** Array that objects are stored in. */
    protected T[] objects;

    /** Number of objects stored in this list. */
    protected int size;

    /**
     * Constructor, initializes an empty list.
     */
    @SuppressWarnings("unchecked")
    public ArrayList() {
        objects = (T[]) new Object[INIT_SIZE];
        size = 0;
    }
    
    /**
     * Add an object to the end of this array list. If the specified object is
     * null, it is not added to the list.
     * 
     * @param object object to add
     */
    public final void add(final T object) {
        add(object, size);
    }

    /**
     * Add an object to the specified location of this array list. Objects that
     * are at the specified index and higher are shifted up by 1 index in order
     * to make room for the new object. If the specified object is null or the
     * index is not valid the list is unchanged.
     * 
     * @param object object to add
     * @param index index to add object
     */
    public final void add(final T object, final int index) {
        if (index < 0 || index > size) {
            throw new IllegalArgumentException("Invalid list index: " + index);
        }
        ArrayList<T> singleList = new ArrayList<T>();
        singleList.objects[0] = object;
        singleList.size++;
        addList(singleList, index);
    }

    /**
     * Add the objects in the specified array list to the end of this array
     * list. The objects should be added in the order that they are stored. If
     * the array of objects is null or empty this array list is unchanged.
     * 
     * @param newObjects list of objects to add
     */
    public final void addList(final ArrayList<T> newObjects) {
        if ( newObjects == null) {
            throw new IllegalArgumentException("Can't add null list.");
        }
        addList(newObjects, size);
    }

    /**
     * Add the objects in the specified array list to the specified location in
     * this array list. The objects should be added in the order that they are
     * stored. Objects that at the specified index and higher are shifted up by
     * the number of objects that are added in order to make room for the new
     * objects. If the array of objects is null or empty this array list is
     * unchanged.
     * 
     * @param newObjects list of objects to add
     * @param index to add list of objects
     */
    public final void addList(final ArrayList<T> newObjects, final int index) {
        if (index < 0 || index > size || newObjects == null) {
            throw new IllegalArgumentException("Invalid list index: " + index);
        }
        if (newObjects == null) {
            throw new IllegalArgumentException("Invalid list: " + newObjects);
        }
        int newSize = newObjects.size();
        if (size + newSize >= objects.length) {
            int n = (int) (Math.log((size + newSize) / objects.length) / Math
                    .log(2.0)) + 1;
            expandObjects((int) (objects.length * Math.pow(2.0, n)));
        }
        for (int i = size - 1; i >= index; i--) {
            objects[i + newSize] = objects[i];
        }
        for (int i = 0; i < newSize; i++) {
            objects[i + index] = newObjects.get(i);
        }
        size += newSize;
    }

    /**
     * Remove the highest indexed object of this array list. If after removing
     * the object the number of objects stored in the array is less than one
     * quarter of the array length and the array length is greater than its
     * initial length, then the array is halved in order to save memory. If the
     * list is empty, the list is unchanged and null is returned.
     * 
     * @return the object that was removed
     */
    public final T remove() {
        if (size == 0) {
            throw new IllegalArgumentException("Can't remove from empty list");
        }
        return remove(size - 1);
    }

    /**
     * Remove the object at the specified index of this array list. All objects
     * that have a higher index are shifted down one index in the array to fill
     * the gap created by removing the object. If after removing the object the
     * number of objects stored in the array is less than one quarter of the
     * array length and the array length is greater than its initial length,
     * then the array is halved in order to save memory. If the index is not
     * valid the list is unchanged and null is returned.
     * 
     * @param index the index of the object to remove.
     * @return the object that was removed
     */
    public final T remove(final int index) {
        if (index < 0 || index >= size) {
            throw new IllegalArgumentException("Invalid list index: " + index);
        }
        return removeList(index, index + 1).get(0);
    }

    /**
     * Remove objects between and including the specified start index and the
     * end of the list. If after removing the objects the number of objects
     * stored in the array is less than one quarter of the array length and the
     * array length is greater than its initial length, then the array is halved
     * in order to save memory. If the start index is not valid the list is
     * unchanged and null is returned.
     * 
     * @param startIndex index of first object to remove
     * @return the list of objects that were removed
     */
    public final ArrayList<T> removeList(final int startIndex) {
        if (startIndex < 0 || size == 0) {
            throw new IllegalArgumentException("invalid start index: " + startIndex);
        }
        return removeList(startIndex, size);
    }

    /**
     * Remove objects between and including the specified start index and
     * excluding the specified end index. All objects that have a higher index
     * than the highest indexed object that is removed are shifted down by the
     * number of objects that are removed to fill the gap created by removing
     * the objects. If after removing the objects the number of objects stored
     * in the array is less than one quarter of the array length and the array
     * length is greater than its initial length, then the array is halved in
     * order to save memory. If the index of any of the objects to be removed is
     * not valid the list is unchanged and null is returned.
     * 
     * @param startIndex index of first object to remove
     * @param endIndex index of the last object to remove (exclusive)
     * @return the list of objects that were removed
     */
    public final ArrayList<T> removeList(final int startIndex,
            final int endIndex) {
        ArrayList<T> removed = null;
        int removeSize = (endIndex - startIndex);
        if (startIndex < 0 || startIndex >= size || endIndex < 0
                || endIndex >= size || startIndex >= endIndex) {
            throw new IllegalArgumentException("Invalid indices: " + startIndex
                    + ", " + endIndex);
        }
        removed = new ArrayList<T>();
        for (int i = startIndex; i < endIndex; i++) {
            removed.add(objects[i]);
        }
        for (int i = endIndex; i < size; i++) {
            objects[i - removeSize] = objects[i];
        }
        for (int i = 1; i <= removeSize; i++) {
            objects[size - i] = null;
        }
        size -= removeSize;
        if (size < objects.length * MIN_CAPACITY) {
            contractObjects();
        }
        return removed;
    }

    /**
     * Double the size of the array of objects.
     */
    @SuppressWarnings("unchecked")
    private void expandObjects(int newSize) {
        T[] newObjects = (T[]) new Object[newSize];
        System.arraycopy(objects, 0, newObjects, 0, objects.length);
        objects = newObjects;
    }

    /**
     * Halve the size of the array of objects.
     */
    @SuppressWarnings("unchecked")
    private void contractObjects() {
        T[] newObjects = (T[]) new Object[objects.length / 2];
        System.arraycopy(objects, 0, newObjects, 0, newObjects.length);
        objects = newObjects;
    }

    /**
     * Returns the object in the list at the specified index. If the index is
     * invalid returns null.
     * 
     * @param index index of element to get
     * @return object at specified index, or null if index is invalid
     */
    public final T get(final int index) {
        T object = null;
        if (index >= 0 && index < size) {
            object = objects[index];
        }
        return object;
    }

    /**
     * Randomly reorders the objects in this list by repeatedly choosing two
     * indices and swapping the two objects.
     */
    public final void shuffle() {
        Random rand = new Random();
        for (int i = 0; i < size - 1; i++) {
            swap(i, rand.nextInt(size - i) + i);
        }
    }

    /**
     * Swap the two objects at the specified indices. If the indices are invalid
     * the list is unchanged.
     * 
     * @param index1 index of the first object
     * @param index2 index of the second object
     */
    protected final void swap(final int index1, final int index2) {
        if (index1 >= 0 && index1 < size && index2 >= 0 && index1 < size
                && index1 != index2) {
            T temp = objects[index2];
            objects[index2] = objects[index1];
            objects[index1] = temp;
        }
    }

    /**
     * Returns the number of objects currently stored in this array list.
     * 
     * @return the number of objects in this array list
     */
    public final int size() {
        return size;
    }

    /**
     * Returns a string representation of this array list.
     * 
     * @return a string representation of this array list
     */
    public final String toString() {
        String string = "[";
        for (int i = 0; i < size - 1; i++) {
            string += objects[i] + ", ";
        }
        if (size > 0) {
            string += objects[size - 1];
        }
        string += "]";
        return string;
    }
}
