/**
 * Player.java
 * 2/9/11
 */

import java.awt.Image;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;

/**
 * Represents the player in a simple platform game.
 * 
 * @author CPSC170A
 * 
 */
public class Player extends Sprite {

    /** Whether player is on a platform. */
    private boolean onPlatform;

    /** Images to animate the player. */
    private Image[] images;

    /** Number of frames since animation looped. */
    private int frameCnt;

    /** Direction the player is facing. */
    private int facing;

    /** If player is facing right. */
    private static final int RIGHT = 0;

    /** If player is facing left. */
    private static final int LEFT = 1;

    /** Index of right standing image. */
    private static final int STAND_RIGHT = 0;

    /** Index of left standing image. */
    private static final int STAND_LEFT = 1;

    /** Index of right running image. */
    private static final int RUN_RIGHT = 2;

    /** Index of left running image. */
    private static final int RUN_LEFT = 3;

    /** Number of Images. */
    private static final int NUM_IMAGES = 4;

    /**
     * Constructor, initializes a new ball.
     * 
     * @param initX initial x location
     * @param initY initial y location
     * @param initWidth initial width
     * @param initHeight initial height
     */
    public Player(final double initX, final double initY,
            final double initWidth, final double initHeight) {
        super(initX, initY, initWidth, initHeight, 0.0, 0.0, null);
        onPlatform = false;
        images = new Image[NUM_IMAGES];
        try {
            images[STAND_RIGHT] = ImageIO
                    .read(new File("PlayerStandRight.png"));
            images[STAND_LEFT] = ImageIO.read(new File("PlayerStandLeft.png"));
            images[RUN_RIGHT] = ImageIO.read(new File("PlayerRunRight.png"));
            images[RUN_LEFT] = ImageIO.read(new File("PlayerRunLeft.png"));
            setImage(images[STAND_RIGHT]);
        } catch (IOException e) {
            images = null;
        }
        frameCnt = 0;
        facing = RIGHT;
    }

    /**
     * Update the location, velocity, and image of the player.
     * 
     * @param keyboardState current game keyboard state
     * @param platforms list of platforms
     */
    public final void update(final KeyboardState keyboardState,
            final PlatformList platforms) {
        updateVelocity(keyboardState);
        updateImage();
        updateLocation(platforms);
        frameCnt = (frameCnt + 1) % Settings.RUN_LOOP_LENGTH;
    }

    /**
     * Update the velocity of the player.
     * 
     * @param keyboardState current game keyboard state
     */
    private void updateVelocity(final KeyboardState keyboardState) {
        if (keyboardState.isPressed(KeyboardState.LEFT)) {
            deltaX = Settings.INIT_PLAYER_VEL * -1.0;
            facing = LEFT;
        } else if (keyboardState.isPressed(KeyboardState.RIGHT)) {
            deltaX = Settings.INIT_PLAYER_VEL;
            facing = RIGHT;
        } else {
            deltaX = 0.0;
        }
        if (keyboardState.isPressed(KeyboardState.UP) && onPlatform) {
            deltaY -= Settings.JUMP;
        }
        deltaY += Settings.GRAVITY;

    }

    /**
     * Update the image of this player based on state.
     */
    private void updateImage() {
        if (facing == RIGHT) {
            if (!onPlatform
                    || (frameCnt < Settings.RUN_LOOP_LENGTH / 2
                            && deltaX > 0.0)) {
                setImage(images[RUN_RIGHT]);
            } else {
                setImage(images[STAND_RIGHT]);
            }
        } else {
            if (!onPlatform
                    || (frameCnt < Settings.RUN_LOOP_LENGTH / 2
                            && deltaX < 0.0)) {
                setImage(images[RUN_LEFT]);
            } else {
                setImage(images[STAND_LEFT]);
            }
        }
    }

    /**
     * Update the location of this player.
     * 
     * @param platforms list of platforms
     */
    private void updateLocation(final PlatformList platforms) {
        Sprite platform;
        double topOverlap, bottomOverlap, leftOverlap, rightOverlap;
        x += deltaX;
        y += deltaY;
        onPlatform = false;
        for (int i = 0; i < platforms.size(); i++) {
            platform = platforms.get(i);
            if (intersect(platforms.get(i))) {
                topOverlap = topOverlap(platform);
                bottomOverlap = bottomOverlap(platform);
                leftOverlap = leftOverlap(platform);
                rightOverlap = rightOverlap(platform);
                if (topOverlap < bottomOverlap && topOverlap < leftOverlap
                        && topOverlap < rightOverlap && deltaY > 0.0) {
                    y = platform.y - height;
                    deltaY = 0.0;
                    onPlatform = true;
                }
            }
        }
    }

}
